/**
 *  This program simulates atoms diffusing in a closed container.
 *  Atoms are places at the center of a square grid.  The number of
 *  atoms can be given as a command-line parameter; if not, 1000 is
 *  used as the default number.  The atoms diffuse from the center.
 *  That is, in each time step, each atom takes a step in one of 
 *  eight possible directions, at random.  The result is shown in
 *  a window in which the number of atoms at a given point corresponds
 *  to the brightness of a small square.  The process is repeated
 *  indefinitely until the user closes this window.  This program uses
 *  the Mosaic class and also depends on classes used by Mosaic.
 */

import java.awt.Point;
 
public class Diffusion {

   private static final int ROWS = 50;  // number of rows in Mosaic
   private static final int COLS = 50;  // number of columns in Mosaic
   private static final int SIZE = 10;  // size of each square in Mosaic


   private static Point[] atomPos;   // (x,y) positions of all the atoms
   
   private static int[][] atomCount; // atomCount[r][c] is the number of
                                     //  of atoms in row r, column c
   
   
   public static void main(String[] args) {
      
      int atoms = 1000; // Number of atoms; 1000 is a default value.
      
      if (args.length > 0) {  // get atomCount from command-line argument
         try {
            atoms = Integer.parseInt(args[0]);
            if (atoms <= 0)
               throw new NumberFormatException("Number must be positive");
         }
         catch (NumberFormatException e) {
            System.out.println("Illegal command-line argument: " + args[0]);
         }
      }
      
      System.out.println("Using " + atoms + " atoms;");
      
      atomPos = new Point[atoms];
      for (int i = 0; i < atoms; i++)
         atomPos[i] = new Point(ROWS/2,COLS/2);
      atomCount = new int[ROWS][COLS];
      atomCount[ROWS/2][COLS/2] = atoms;
      
      Mosaic.open(ROWS,COLS,SIZE,SIZE);
      
      while (Mosaic.isOpen()) {
         for (int i = 0; i < atoms; i++)
            moveAtom(i);
      }
      
      System.exit(0);
      
   } // end main()
   
   
   private static void moveAtom(int atomNumber) {
         // Move the atom  number atomNumber one square in one of
         // the eight possible random directions, chosen at random.

      int oldRow = atomPos[atomNumber].x; // Current position of atom.
      int oldCol = atomPos[atomNumber].y;

      int newRow = oldRow; // These will give the new position of atom.
      int newCol = oldCol;

      switch ( (int)(8*Math.random()) ) { 
                           // Move newRow,newCol in a random direction.
         case 0: newRow++; break;
         case 1: newRow--; break;
         case 2: newCol++; break;
         case 3: newCol--; break;
         case 4: newRow++; newCol++; break;
         case 5: newRow++; newCol--; break;
         case 6: newRow--; newCol++; break;
         case 7: newRow--; newCol--; break;
      }

      if (newRow < 0 || newRow >= ROWS || newCol < 0 || newCol >= COLS) {
           // New position lies outside the mosaic.  Handle this by
           // simply not allowing the atom to move.
         return;
      }
      
      atomPos[atomNumber].x = newRow;  // Change the position of the atom.
      atomPos[atomNumber].y = newCol;
      
      atomCount[oldRow][oldCol]--;   // Adjust atom count at old position.
      setColorLevel(oldRow,oldCol);  // Adjust color at old position.
      atomCount[newRow][newCol]++;   // Adjust atom count at new position.
      setColorLevel(newRow,newCol);  // Adjust color at new position.
      
   } // end moveAtom()

   
   private static final int[] colorLevels = 
     { 0, 75, 100, 125, 150, 170, 190, 210, 225, 235, 240, 245, 250 };

   private static void setColorLevel(int row, int col) {
         // Set the color level of the Mosaic square in the given
         // row and column to reflect the number of atoms in that square
      int ct = atomCount[row][col];
      if (ct >= colorLevels.length)
         Mosaic.setColor(row,col,0,255,0);
      else
         Mosaic.setColor(row,col,0,colorLevels[ct],0);
   
   } // end setColorLevel()

} // end class Diffusion
